/*
**	2007 Leonid Tsinitskiy (LionTs)
**	Simple CD player
**		this sample covers:
**			- using CD_... commands to play audioCD
**			- making skinnable UI with help of CR Pack
**			- draggable CD Player window
**		http://www.Lion-Ts.com
*/

/*
**	include code
*/
include <custres.wdl>;	//use custom resource functions

/*
**	bind resources
*/
bind <logolite.gsr>;	//include custom resources in published game
bind <resource.gsr>;	//skin1 files
bind <levels.gsr>;	//skin2 files

/*
**	defines, variables, strings, texts, etc.
*/
plugindir = ".";

var video_mode = 7; // 800x600
var video_depth = 32; // 32 bit mode
var video_screen = 2;  //windowed mode
var wndf=0;	//for Alt+Tab processing

function* old_on_enter;	//Alt+Enter handler

bmap* MouseMap;	//cursor

bmap* pause_map;	//set of images for PLAY / PAUSE button
bmap* pausedn_map;
bmap* play_map;
bmap* playdn_map;

panel* wait_pan;		// logo panel placeholder

string level_wmb = <ACDPlayer.wmb>;
string res_demo  = "Custom Resources for 3DGS A6.4+\nDemo #8 by LionTs,\nLook at main.wdl and custres.wdl for details\n"; 
string help_str = "3DGS AudioCD Player       S-next Skin    F10-exit"; 
string WndCaption = "CR Pack Demo #7 by LionTs";
string strChoice;	//selected image name
string strTemp;	//temp string
string strTemp1;	//temp string

font skinc_font = "font.tga", 16, 23;	//help string font
font digit_font0 = <digfont.pcx>,12,16;	//numbers font

text res_demo_txt	// displays "Demo..."
{
	pos_x = 25;
	pos_y = 0;
	layer = 100;
	font = skinc_font;
	string = res_demo;
}
text help_txt		// displays help string
{
	pos_x = 0;
	pos_y = 575;
	layer = 100;
	font = skinc_font;
	string = help_str;
}

string PlayerPanDef;	//string to build CD Player panel definition
var CurrentSkin = -1;	//current skin of CD Player
var CurrentTrack = 0;	//current CD track

text skins	//skins placeholder
{
	strings = 2;
	string = "resource.gsr", "levels.gsr";
}
/*
**	Utility functions
*/
//our Alt+Enter handler
function new_on_enter
{
	old_on_enter();	//call old handler
	if (video_screen==2){	//setup window params
		video_window(nullvector, nullvector, 113, WndCaption);	//no resize & new wnd caption
	}
}
//exit function
function game_fin()
{
	proc_kill(4);					//for a case of logical errors in code
	CR_Close("logolite.gsr", On);	//unload all splash screens
	if (CurrentSkin != -1){	//special case of a first call
		CR_Close(skins.string[CurrentSkin], On);
	}
	sys_exit("");
}
//center the image
function center_image
{
	wait_pan.pos_x = (screen_size.x - bmap_width(wait_pan.bmap))/2;	//center hor
	wait_pan.pos_y = (screen_size.y - bmap_height(wait_pan.bmap))/2;	//center vert
}
//show and track the mouse
function mouse_on // switches the mouse on
{  
	MOUSE_MAP = MouseMap;
	MOUSE_MODE = 2;
	while (1){  
		MOUSE_POS.X = mouse_cursor.X;
		MOUSE_POS.Y = mouse_cursor.Y;
		wait(1);
	}
}
//change skin hotkey handler
function change_skin
{
	var fh;
	var n = 0;
	var TempPos[2] = -1, -1;
	
	if (wait_pan != NULL){
		wait_pan.visible = off;	//hide CD player
		TempPos.x = wait_pan.pos_x;	//save position
		TempPos.y = wait_pan.pos_y;
		bmap_remove(pause_map);	//remove PLAY/PAUSE buttons
		bmap_remove(pausedn_map);
		bmap_remove(play_map);
		bmap_remove(playdn_map);
		pan_remove(wait_pan);	//remove panel
	}
	if (CurrentSkin != -1){	//special case of a first call
		CR_Close(skins.string[CurrentSkin], On);
	}
	
	CurrentSkin = cycle(CurrentSkin + 1, 0, 2);	//cycle to the next skin
	CR_Open(skins.string[CurrentSkin], "", On);	//open skin file
	CR_LoadAll(skins.string[CurrentSkin], On);	//load skin files
	
	pause_map = bmap_create("btnpause.tga");	//create images for PLAY/PAUSE buton
	pausedn_map = bmap_create("btnpausedn.tga");
	play_map = bmap_create("btnplay.tga");
	playdn_map = bmap_create("btnplaydn.tga");
	
	CR_ExtractFile(skins.string[CurrentSkin], "", "skin.txt", On);	//extraxt skin definition
	fh = file_open_game("skin.txt");	//open skin definition file
	str_cpy(PlayerPanDef, "");	//empty string
	while (1){	//load skin definition
		n = file_str_read(fh, strTemp);
		if (n == -1){
			break;
		}
		str_cat(PlayerPanDef, strTemp);
	}
	file_close(fh);
	file_delete("skin.txt");	//don't forget to remove our skin definition from the game folder!
	
	wait_pan = pan_create(PlayerPanDef, 1);	//create "skinned" CD Player panel
	
	if (Temppos.x == -1){	//center CD Player if this is the first run
		center_image();
	}else{	//or restore its position
		wait_pan.pos_x = TempPos.x;
		wait_pan.pos_y = TempPos.y;
	}
	
	if (CurrentTrack != 0){	//change button PLAY / PAUSE for a new one
		pan_setbmap(wait_pan, 3, 0.0, pausedn_map);
		pan_setbmap(wait_pan, 3, 0.1, pause_map);
		pan_setbmap(wait_pan, 3, 0.2, pause_map);
	}else{	//change button to PLAY
		pan_setbmap(wait_pan, 3, 0.0, playdn_map);
		pan_setbmap(wait_pan, 3, 0.1, play_map);
		pan_setbmap(wait_pan, 3, 0.2, play_map);
	}
	
	wait_pan.visible = on;	//and show
}
//CD player panel buttons handler
function BtnPressed(BtnNumber, PanelPointer)
{
	if (BtnNumber==1){	//play / pause button
		if (CurrentTrack == 0){	//PLAY pressed
			cd_start();	//resume playing
		}else{	//PAUSE pressed
			cd_pause();	//pause playing
		}
	}else{	//track number button pressed
		CurrentTrack = cd_play(BtnNumber - 1, BtnNumber - 1);	//play selected track only
	}
}
//CD Player window on_click handler (move window)
function move_window(PanelPointer)
{
	var TempPos;
	var Diff[2];
	
	vec_set(TempPos, mouse_cursor);	//save pointer
	Diff.x = TempPos.x - wait_pan.pos_x;	//calculate click offset
	Diff.y = TempPos.y - wait_pan.pos_y;
	while (mouse_left == 1){	//move window while mouse pressed
		wait_pan.pos_x = mouse_cursor.x - Diff.x;
		wait_pan.pos_y = mouse_cursor.y - Diff.y;
		wait(1);
	}
}
/*
**	Starters
*/
//obtain CD track number
// made this function for a case
// that track may change somehow without our CD Player (with button on CD drive, for example)
function get_track_startup()
{
	while(1){	//do it forever
		CurrentTrack = cd_track();	//save current song
		wait(1);
	}
}
//change play/pause button
function change_pp_startup()
{
	var TmpCurrentTrack;
	
	while(1){
		while(CurrentSkin == -1){	//wait for CD Palyer panel 
			wait(1);
		}
		TmpCurrentTrack = CurrentTrack;	//save current song
		if (TmpCurrentTrack != 0){	//change button to PAUSE
			pan_setbmap(wait_pan, 3, 0.0, pausedn_map);
			pan_setbmap(wait_pan, 3, 0.1, pause_map);
			pan_setbmap(wait_pan, 3, 0.2, pause_map);
		}else{	//change button to PLAY
			pan_setbmap(wait_pan, 3, 0.0, playdn_map);
			pan_setbmap(wait_pan, 3, 0.1, play_map);
			pan_setbmap(wait_pan, 3, 0.2, play_map);
		}
		while(TmpCurrentTrack == CurrentTrack){	//wait for a new track selection
			wait(1);
		}
		wait(1);	//this line for a case of possible logical errors: TmpCurrentTrack != CurrentTrack allways
	}
}
//pause game on loose focus
function pause_game_at_alt_tab_startup()
{
	while (1)
	{
		if (window_focus == 0){
			if (!wndf){
				wndf=1;
				camera.visible=off;	//for no cpu usage
				fps_max=4;
				freeze_mode = 1;
			}
		}else{
			if (wndf){
				freeze_mode = 0;
				fps_max=40;
				camera.visible=on;
				wndf=0;
			}
		}
		wait(1);
	}
}
//change Alt+Enter handler at start
function save_on_enter_handler_startup
{
	old_on_enter=on_enter;	//save old handler
	on_enter=new_on_enter;	//assign new handler
}

/*
**	main entrypoint
*/
function main()
{
	//black background
	sky_color.red = 10;
	sky_color.green = 10;
	sky_color.blue = 10;
	//change window caption
	if (video_screen==2){
		
		video_window(nullvector, nullvector, 113, WndCaption);	//no resize & new wnd caption
	}
	camera.visible=off;
	fps_max=4;
	freeze_mode = 1;

	str_cpy(delimit_str,"~");	//this char may not be used in the skin file

	CR_Open("logolite.gsr", "", On);	//open custom resource
	CR_LoadAll("logolite.gsr", On);	//load all splash screens
	wait_pan = pan_create("bmap = logolite.tga; alpha=50;", 1000);
	center_image();
	wait_pan.visible = on;
	wait(3);
	clip_size = 0; // show all the triangles for all the models
	fps_max = 40; // lock the frame rate to 30 fps
	fps_min=16;
	level_load(level_wmb);	//load compiled level
	wait (3); // wait for the level to be loaded
	
	wait(-1);	//delay to display Conitec's logo for a 1 sec at least
	
	//display level in view
	camera.visible=on;
	res_demo_txt.visible = on;
	help_txt.visible = on;
	freeze_mode = 0;
	wait(3);
	freeze_mode = 1;
	//dissolving logo
	wait_pan.transparent = on;
	while (wait_pan.alpha > 0)
	{
		wait_pan.alpha = max(wait_pan.alpha - 7 * time_step, 0);
		wait(1);
	}
	wait_pan.visible = off;
	
	pan_remove(wait_pan);	// remove splashscreen

	wait_pan = pan_create("bmap = liontslogo.tga; alpha=100;", 1000);
	center_image();
	wait_pan.visible = on;
	wait(-3);

	var j;
	j = wait_pan.scale_y / wait_pan.scale_x;
	//disappearing logo
	while(wait_pan.scale_x > 0.1){
		wait_pan.scale_x = max(0.1, wait_pan.scale_x - 0.1 * time_step);
		wait_pan.scale_y = wait_pan.scale_x * j;
		wait_pan.pos_x = (screen_size.x - bmap_width(wait_pan.bmap) * wait_pan.scale_x)/2;
		wait_pan.pos_y = (screen_size.y - bmap_height(wait_pan.bmap) * wait_pan.scale_y)/2;
		wait(1);
	}
	wait_pan.visible = off;
	
	pan_remove(wait_pan);	// remove splashscreen
	wait_pan = NULL;	//inform change_skin() function: this is a first call to it
	
	CR_UnLoadFile("logolite.gsr", "logolite.tga", On);	//unload splash screen
	CR_UnLoadFile("logolite.gsr", "liontslogo.tga", On);	//unload splash screen
	
	fps_max=40;
	freeze_mode = 0;
	MouseMap = bmap_create("cursor.tga");	//change mouse cursor
	mouse_on();	//show and track the mouse
	
	master_vol = 50;	//set 50% master volume
	cdaudio_vol = 100;	//and max CD Audio volume
	
	change_skin();	//show CD Player
	key_set(31, change_skin);	//set hotkey to change skin
}

/*
**	Key bindings
*/
//one way to exit
on_close = game_fin;
on_f10 = game_fin;
on_esc = NULL;